# -*- coding: utf-8 -*-


class ShopService(object):
    ShopService_REQ_TYPE = 3
    ShopService_RES_TYPE = 3

    def __init__(self):
        pass

    def getProduct(self, shopId, productId, language="zh-TW", country="TW"):
        sqrd = [128, 1, 0, 1, 0, 0, 0, 10, 103, 101, 116,
                80, 114, 111, 100, 117, 99, 116, 0, 0, 0, 0]
        sqrd += [11, 0, 2, 0, 0, 0, len(shopId)]  # e.g. stickershop
        for value in shopId:
            sqrd.append(ord(value))
        sqrd += [11, 0, 3, 0, 0, 0, len(productId)]
        for value in productId:
            sqrd.append(ord(value))
        sqrd += [12, 0, 4]
        sqrd += [11, 0, 1, 0, 0, 0, len(language)]
        for value in language:
            sqrd.append(ord(value))
        sqrd += [11, 0, 2, 0, 0, 0, len(country)]
        for value in country:
            sqrd.append(ord(value))
        sqrd += [0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd)

    def getProductsByAuthor(self, authorId, productType=1):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('getProductsByAuthor') + [0, 0, 0, 0]
        sqrd += [12, 0, 2]
        sqrd += [8, 0, 1] + self.getIntBytes(productType)
        sqrd += [11, 0, 2] + self.getStringBytes(authorId)
        sqrd += [8, 0, 3] + self.getIntBytes(0)
        sqrd += [8, 0, 4] + self.getIntBytes(100)
        sqrd += [2, 0, 6, int(True)]
        sqrd += [0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd)

    def getStudentInformation(self):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('getStudentInformation') + [0, 0, 0, 0]
        sqrd += [12, 0, 2]
        sqrd += [0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd)

    def canReceivePresent(self, shopId, productId, recipientMid):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('canReceivePresent') + [0, 0, 0, 0]
        sqrd += [11, 0, 2] + self.getStringBytes(shopId)
        sqrd += [11, 0, 3] + self.getStringBytes(productId)
        sqrd += [12, 0, 4]
        sqrd += [11, 0, 1] + self.getStringBytes('zh_TW')  # language
        sqrd += [11, 0, 2] + self.getStringBytes('TW')  # country
        sqrd += [0]
        sqrd += [11, 0, 5] + self.getStringBytes(recipientMid)
        sqrd += [0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd)

    def getOwnedProductSummaries(self, shopId, offset=0, limit=200, language='zh_TW', country='TW'):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('getOwnedProductSummaries') + [0, 0, 0, 0]
        sqrd += [11, 0, 2] + self.getStringBytes(shopId)
        sqrd += [8, 0, 3] + self.getIntBytes(offset)
        sqrd += [8, 0, 4] + self.getIntBytes(limit)
        sqrd += [12, 0, 5]
        sqrd += [11, 0, 1] + self.getStringBytes(language)
        sqrd += [11, 0, 2] + self.getStringBytes(country)
        sqrd += [0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd)

    def getShowcaseV3(self, productType: int, showcaseType: int, subType: int, continuationToken: str = None, limit: int = 20):
        """
        - productType
            STICKER(1),
            THEME(2),
            STICON(3);
        - showcaseType
            UNPURCHASED(1),
            SUBSCRIPTION(2);
        - subType
            GENERAL(0),
            CREATORS(1),
            STICON(2);
        """
        params = [
            [12, 1, [  # Shop_ShowcaseRequest
                [8, 1, productType],
                [8, 2, showcaseType],
                [8, 3, subType],
                [11, 4, continuationToken],
                [8, 5, limit],
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            'getShowcaseV3', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getProductV2(self, productType: int, productId: str, carrierCode: str = "", saveBrowsingHistory: bool = True):
        params = [
            [12, 2, [
                [8, 1, productType],
                [11, 2, productId],
                [11, 3, carrierCode],
                [2, 4, saveBrowsingHistory],
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            'getProductV2', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getProductByVersion(self, shopId: str, productId: str, productVersion: int, language: str = 'zh_TW', country: str = 'TW'):
        params = [
            [11, 2, shopId],
            [11, 3, productId],
            [10, 4, productVersion],
            [12, 5, [
                [11, 1, language],
                [11, 2, country],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getProductByVersion', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def placePurchaseOrderForFreeProduct(self, shopId: str, productId: str, recipientMid: str, price: str, amount: str, priceString: str, enableLinePointAutoExchange: bool = True, language: str = 'zh_TW', country: str = 'TW', presentAttributes: dict = {}):
        params = [
            [12, 2, [
                [11, 1, shopId],
                [11, 2, productId],
                [11, 5, recipientMid],
                [12, 11, [
                    [11, 1, price],
                    [11, 2, amount],
                    [11, 5, priceString],
                ]],
                [2, 12, enableLinePointAutoExchange],
                [12, 21, [
                    [11, 1, language],
                    [11, 2, country],
                ]],
                [13, 31, [11, 11, presentAttributes]],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'placePurchaseOrderForFreeProduct', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def placePurchaseOrderWithLineCoin(self, shopId: str, productId: str, recipientMid: str, price: str, amount: str, priceString: str, enableLinePointAutoExchange: bool = True, language: str = 'zh_TW', country: str = 'TW', presentAttributes: dict = {}):
        params = [
            [12, 2, [
                [11, 1, shopId],
                [11, 2, productId],
                [11, 5, recipientMid],
                [12, 11, [
                    [11, 1, price],
                    [11, 2, amount],
                    [11, 5, priceString],
                ]],
                [2, 12, enableLinePointAutoExchange],
                [12, 21, [
                    [11, 1, language],
                    [11, 2, country],
                ]],
                [13, 31, [11, 11, presentAttributes]],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'placePurchaseOrderWithLineCoin', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def placePurchaseOrderWithIAP(self, shopId: str, productId: str, recipientMid: str, price: str, amount: str, priceString: str, enableLinePointAutoExchange: bool = True, language: str = 'zh_TW', country: str = 'TW', presentAttributes: dict = {}):
        params = [
            [12, 2, [
                [11, 1, shopId],
                [11, 2, productId],
                [11, 5, recipientMid],
                [12, 11, [
                    [11, 1, price],
                    [11, 2, amount],
                    [11, 5, priceString],
                ]],
                [2, 12, enableLinePointAutoExchange],
                [12, 21, [
                    [11, 1, language],
                    [11, 2, country],
                ]],
                [13, 31, [11, 11, presentAttributes]],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'placePurchaseOrderWithIAP', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getOwnedProducts(self, shopId: str, offset: int = 0, limit: int = 20, language: str = 'zh_TW', country: str = 'TW'):
        params = [
            [11, 2, shopId],
            [8, 3, offset],
            [8, 4, limit],
            [12, 5, [
                [11, 1, language],
                [11, 2, country],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getOwnedProducts', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getPurchasedProducts(self, shopId: str, offset: int = 0, limit: int = 20, language: str = 'zh_TW', country: str = 'TW'):
        params = [
            [11, 2, shopId],
            [8, 3, offset],
            [8, 4, limit],
            [12, 5, [
                [11, 1, language],
                [11, 2, country],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getPurchasedProducts', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getReceivedPresents(self, shopId: str, offset: int = 0, limit: int = 20, language: str = 'zh_TW', country: str = 'TW'):
        params = [
            [11, 2, shopId],
            [8, 3, offset],
            [8, 4, limit],
            [12, 5, [
                [11, 1, language],
                [11, 2, country],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getReceivedPresents', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getSentPresents(self, shopId: str, offset: int = 0, limit: int = 20, language: str = 'zh_TW', country: str = 'TW'):
        params = [
            [11, 2, shopId],
            [8, 3, offset],
            [8, 4, limit],
            [12, 5, [
                [11, 1, language],
                [11, 2, country],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getSentPresents', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def notifyProductEvent(self, shopId: str, productId: str, productVersion: int, productEvent: int):
        params = [
            [11, 2, shopId],  # sticonshop
            [11, 3, productId],  # 1
            [10, 4, productVersion],  # 3
            [10, 5, productEvent],  # 16
        ]
        sqrd = self.generateDummyProtocol(
            'notifyProductEvent', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getProductValidationScheme(self, shopId: str, productId: str, productVersion: int):
        params = [
            [11, 2, shopId],
            [11, 3, productId],
            [10, 4, productVersion],
        ]
        sqrd = self.generateDummyProtocol(
            'getProductValidationScheme', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def validateProduct(self, shopId: str, productId: str, productVersion: int, key: str, offset: int, size: int, authCode: str):
        params = [
            [11, 2, shopId],
            [11, 3, productId],
            [10, 4, productVersion],
            [12, 5, [
                [12, 1, [
                    [11, 10, key],
                    [10, 11, offset],
                    [10, 12, size],
                ]],
                [11, 10, authCode]
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'validateProduct', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getProductsByBillingItemId(self, shopId: str, billingItemIds: list, language: str = 'zh_TW', country: str = 'TW'):
        params = [
            [11, 2, shopId],
            [15, 3, [11, billingItemIds]]
            [12, 4, [
                [11, 1, language],
                [11, 2, country],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getProductsByBillingItemId', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getUpdates(self, shopId: str, language: str = 'zh_TW', country: str = 'TW'):
        params = [
            [11, 2, shopId],
            [12, 3, [
                [11, 1, language],
                [11, 2, country],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getUpdates', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def searchProductsV2(self, query: str, productTypes: list, subtypes: list, priceTiers: list, stickerResourceTypes: list, productResourceTypes: list, continuationToken: str, limit: int = 10, deprecatedOffsetForLineSearchServer: int = 0, sortType: int = 0, enableSearchSuggestKeywords: bool = False):
        params = [
            [12, 2, [
                [11, 1, query],
                [14, 2, [8, productTypes]],
                [14, 3, [8, subtypes]],
                [11, 4, continuationToken],
                [8, 5, limit],
                [8, 6, deprecatedOffsetForLineSearchServer],
                [14, 7, [8, priceTiers]],
                [14, 8, [8, stickerResourceTypes]],
                [8, 9, sortType],
                [10, 14, [8, productResourceTypes]],
                [2, 100, enableSearchSuggestKeywords],
                # [12, 101, [
                # Shop_ShopFilter.h
                # ]],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'searchProductsV2', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getAggregatedHomeV2(self, showcaseRequests: list, enableEditorsPickBanner: bool = True, enableAuthorList: bool = True, enableKeywordSticker: bool = True, enableDetailedEditorsPick: bool = True, enableDetailedCategory: bool = True, enableCategoryList: bool = True, enableTagsList: bool = True):
        params = [
            [12, 2, [
                [15, 1, [12, showcaseRequests]],  # Shop_ShowcaseRequest
                [2, 2, enableEditorsPickBanner],
                [2, 3, enableAuthorList],
                [2, 4, enableKeywordSticker],
                [2, 5, enableDetailedEditorsPick],
                [2, 6, enableDetailedCategory],
                [2, 7, enableCategoryList],
                [2, 8, enableTagsList],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getAggregatedHomeV2', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getAggregatedHomeNative(self, productType: int):
        params = [
            [12, 2, [
                [8, 1, productType],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getAggregatedHomeNative', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getDynamicHomeNative(self, productType: int = 1):
        params = [
            [12, 2, [
                [8, 1, productType],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getDynamicHomeNative', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getAggregatedPremiumHome(self, showcaseRequests: list):
        params = [
            [12, 2, [
                [15, 1, [12, showcaseRequests]],  # Shop_ShowcaseRequest
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getAggregatedPremiumHome', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getAggregatedShowcaseV4(self, productType: int, showcaseRequests: list):
        params = [
            [12, 2, [
                [8, 1, productType],
                [15, 2, [12, showcaseRequests]],  # Shop_ShowcaseRequest
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getAggregatedShowcaseV4', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getRecommendationForUser(self, shopId: str, offset: int = 0, limit: int = 20, language: str = 'zh_TW', country: str = 'TW'):
        params = [
            [11, 2, shopId],
            [8, 3, offset],
            [8, 4, limit],
            [12, 5, [
                [11, 1, language],
                [11, 2, country],
            ]],
        ]
        sqrd = self.generateDummyProtocol(
            'getRecommendationForUser', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getRecommendationList(self, productType: int, recommendationType: int, productId: str, offset: int = 0, limit: int = 20, language: str = 'zh_TW', continuationToken: str = None, shouldShuffle: bool = False, includeStickerIds: bool = True):
        params = [
            [12, 2, [
                [11, 1, continuationToken],
                [8, 2, limit],
                [8, 3, productType],
                [8, 4, recommendationType],
                [11, 5, productId],
                # [14, 6, [8, subtypes]],
                [2, 7, shouldShuffle],
                [2, 8, includeStickerIds],
                # [12, 9, shopFilter],
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            'getRecommendationList', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getCategories(self, productType: int, recommendationType: int, productId: str, offset: int = 0, limit: int = 20, language: str = 'zh_TW', continuationToken: str = None, shouldShuffle: bool = False, includeStickerIds: bool = True):
        params = [
            [12, 2, [
                [11, 1, continuationToken],
                [8, 2, limit],
                [8, 3, productType],
                [8, 4, recommendationType],
                [11, 5, productId],
                # [14, 6, [8, subtypes]],
                [2, 7, shouldShuffle],
                [2, 8, includeStickerIds],
                # [12, 9, shopFilter],
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            'getCategories', params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def removeProductFromSubscriptionSlot(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("removeProductFromSubscriptionSlot is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "removeProductFromSubscriptionSlot", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getRecommendOa(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getRecommendOa is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getRecommendOa", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSubscriptionCampaigns(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getSubscriptionCampaigns is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getSubscriptionCampaigns", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def buyMustbuyProduct(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("buyMustbuyProduct is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "buyMustbuyProduct", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSubscriptionPlans(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getSubscriptionPlans is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getSubscriptionPlans", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def previewCustomizedImageText(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("previewCustomizedImageText is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "previewCustomizedImageText", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getBrowsingHistory(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getBrowsingHistory is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getBrowsingHistory", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def deleteAllBrowsingHistory(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("deleteAllBrowsingHistory is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "deleteAllBrowsingHistory", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSubscriptionShowcase(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getSubscriptionShowcase is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getSubscriptionShowcase", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getExperiments(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getExperiments is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getExperiments", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getResourceFile(self):
        params = [
            [12, 2, [
                [12, 1, []], # tagClusterFileRequest
                [2, 2, False], # staging
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            "getResourceFile", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getExperimentsV2(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getExperimentsV2 is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getExperimentsV2", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getAutoSuggestionShowcase(self, productType: int = 1, suggestionType: int = 0):
        """
        - suggestionType:
            NOT_PURCHASED(0),
            SUBSCRIPTION(1);
        """
        params = [
            [12, 2, [
                [8, 1, productType],
                [8, 2, suggestionType]
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            "getAutoSuggestionShowcase", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def sendReportForShop(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("sendReport is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "sendReport", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getOldSticonMapping(self, lastUpdatedTimeMillis: int = 1567749600000):
        params = [
            [12, 2, [
                [10, 1, lastUpdatedTimeMillis]
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            "getOldSticonMapping", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getEditorsPickShowcase(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getEditorsPickShowcase is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getEditorsPickShowcase", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getPurchasedSubscriptions(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getPurchasedSubscriptions is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getPurchasedSubscriptions", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getAuthorList(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getAuthorList is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getAuthorList", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def changeSubscription(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("changeSubscription is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "changeSubscription", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSubscriptionStatus(self):
        params = []
        sqrd = self.generateDummyProtocol(
            "getSubscriptionStatus", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def findRestorablePlan(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("findRestorablePlan is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "findRestorablePlan", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def purchaseSubscription(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("purchaseSubscription is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "purchaseSubscription", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSimilarImageShowcase(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getSimilarImageShowcase is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getSimilarImageShowcase", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def setCustomizedImageText(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("setCustomizedImageText is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "setCustomizedImageText", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSuggestResources(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getSuggestResources is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getSuggestResources", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getProductSummariesInSubscriptionSlots(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception(
            "getProductSummariesInSubscriptionSlots is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getProductSummariesInSubscriptionSlots", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getProductsByCategory(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getProductsByCategory is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getProductsByCategory", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getProductsByTags(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getProductsByTags is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getProductsByTags", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSuggestDictionarySetting(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getSuggestDictionarySetting is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getSuggestDictionarySetting", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSuggestResourcesV2(self, productType: int, productIds: list):
        params = [
            [12, 2, [
                [8, 1, productType],  # 3
                [15, 2, [11, productIds]],  # ['5ac1bfd5040ab15980c9b435']
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            "getSuggestResourcesV2", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_UNIFIED_SHOP_ENDPOINT, sqrd, self.ShopService_RES_TYPE)

    def getAuthorsLatestProducts(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getAuthorsLatestProducts is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getAuthorsLatestProducts", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def addProductToSubscriptionSlot(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("addProductToSubscriptionSlot is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "addProductToSubscriptionSlot", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSubscriptionRecommendations(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getSubscriptionRecommendations is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getSubscriptionRecommendations", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def saveStudentInformation(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("saveStudentInformation is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "saveStudentInformation", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getProductsByTagsV2(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getProductsByTagsV2 is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getProductsByTagsV2", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getTags(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getTags is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getTags", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def produceInteractionEvent(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("produceInteractionEvent is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "produceInteractionEvent", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def restoreSubscription(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("restoreSubscription is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "restoreSubscription", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def verifyBirthdayGiftAssociationToken(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception(
            "verifyBirthdayGiftAssociationToken is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "verifyBirthdayGiftAssociationToken", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getShopPopups(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getPopups is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getPopups", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSubscriptionSlotStatus(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getSubscriptionSlotStatus is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getSubscriptionSlotStatus", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getSubscriptionSlotHistory(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getSubscriptionSlotHistory is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getSubscriptionSlotHistory", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)

    def getProductKeyboardGlobalSetting(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getProductKeyboardGlobalSetting is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getProductKeyboardGlobalSetting", params, self.ShopService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.ShopService_API_PATH, sqrd, self.ShopService_RES_TYPE)



# -*- coding: utf-8 -*-

class ChannelService(object):

    def __init__(self):
        pass

    def issueChannelToken(self, channelId="1341209950"):
        # sqrd = self.DummyProtocol("issueChannelToken", 3, {
        #    1: (11, channelId)
        # }).read()
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('issueChannelToken') + [0, 0, 0, 0]
        sqrd += [11, 0, 1] + self.getStringBytes(channelId)
        sqrd += [0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_CHANNEL_ENDPOINT, sqrd)

    def approveChannelAndIssueChannelToken(self, channelId="1341209950"):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes(
                'approveChannelAndIssueChannelToken') + [0, 0, 0, 0]
        sqrd += [11, 0, 1] + self.getStringBytes(channelId)
        sqrd += [0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_CHANNEL_ENDPOINT, sqrd)

    def getChannelInfo(self, channelId):
        sqrd = [128, 1, 0, 1, 0, 0, 0, 14, 103, 101, 116, 67, 104, 97, 110, 110,
                101, 108, 73, 110, 102, 111, 0, 0, 0, 0, 11, 0, 2, 0, 0, 0, len(channelId)]
        for value in str(channelId):
            sqrd.append(ord(value))
        sqrd += [0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_CHANNEL_ENDPOINT, sqrd)

    def getCommonDomains(self, lastSynced=0):
        sqrd = [128, 1, 0, 1, 0, 0, 0, 16, 103, 101, 116, 67, 111, 109,
                109, 111, 110, 68, 111, 109, 97, 105, 110, 115, 0, 0, 0, 0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_CHANNEL_ENDPOINT, sqrd)

    def issueRequestTokenWithAuthScheme(self, channelId, otpId, authScheme, returnUrl):
        sqrd = [128, 1, 0, 1, 0, 0, 0, 31, 105, 115, 115, 117, 101, 82, 101, 113, 117, 101, 115, 116, 84,
                111, 107, 101, 110, 87, 105, 116, 104, 65, 117, 116, 104, 83, 99, 104, 101, 109, 101, 0, 0, 0, 0]
        sqrd += [11, 0, 1, 0, 0, 0, len(channelId)]
        for value in channelId:
            sqrd.append(ord(value))
        sqrd += [11, 0, 2, 0, 0, 0, len(otpId)]
        for value in otpId:
            sqrd.append(ord(value))
        sqrd += [15, 0, 3, 11, 0, 0, 0, len(authScheme)]
        for mid in authScheme:
            sqrd += [0, 0, 0, len(mid)]
            for value in mid:
                sqrd.append(ord(value))
        sqrd += [11, 0, 4, 0, 0, 0, len(returnUrl)]
        for value in returnUrl:
            sqrd.append(ord(value))
        sqrd += [0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_CHANNEL_ENDPOINT, sqrd)

    def getReturnUrlWithRequestTokenForAutoLogin(self, url, sessionString=None):
        params = [
            [12, 2, [
                [11, 1, url],
                [11, 2, sessionString]
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            'getReturnUrlWithRequestTokenForAutoLogin', params, 4)
        return self.postPackDataAndGetUnpackRespData(self.LINE_CHANNEL_ENDPOINT_V4, sqrd, 4)

    def getWebLoginDisallowedUrl(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getWebLoginDisallowedUrl is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getWebLoginDisallowedUrl", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def updateChannelNotificationSetting(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("updateChannelNotificationSetting is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "updateChannelNotificationSetting", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def updateChannelSettings(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("updateChannelSettings is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "updateChannelSettings", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def syncChannelData(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("syncChannelData is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "syncChannelData", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def getUpdatedChannelIds(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getUpdatedChannelIds is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getUpdatedChannelIds", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def getChannels(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getChannels is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getChannels", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def getDomains(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getDomains is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getDomains", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def revokeAccessToken(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("revokeAccessToken is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "revokeAccessToken", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def approveChannelAndIssueRequestToken(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception(
            "approveChannelAndIssueRequestToken is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "approveChannelAndIssueRequestToken", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def getChannelNotificationSettings(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getChannelNotificationSettings is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getChannelNotificationSettings", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def reserveCoinUse(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("reserveCoinUse is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "reserveCoinUse", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def getApprovedChannels(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getApprovedChannels is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getApprovedChannels", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def issueRequestToken(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("issueRequestToken is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "issueRequestToken", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def issueRequestTokenForAutoLogin(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("issueRequestTokenForAutoLogin is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "issueRequestTokenForAutoLogin", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def getNotificationBadgeCount(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getNotificationBadgeCount is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getNotificationBadgeCount", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def revokeChannel(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("revokeChannel is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "revokeChannel", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def getChannelSettings(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getChannelSettings is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getChannelSettings", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def issueOTP(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("issueOTP is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "issueOTP", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def fetchNotificationItems(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("fetchNotificationItems is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "fetchNotificationItems", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def getFriendChannelMatrices(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getFriendChannelMatrices is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getFriendChannelMatrices", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)

    def getChannelNotificationSetting(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getChannelNotificationSetting is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getChannelNotificationSetting", params, ChannelService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(ChannelService_API_PATH, sqrd, ChannelService_RES_TYPE)



# -*- coding: utf-8 -*-

class AccountAuthFactorEapConnectService(object):
    AAFEC_REQ_TYPE = 3
    AAFEC_RES_TYPE = 3

    def __init__(self):
        pass

    def connectEapAccount(self, authSessionId: str):
        params = [
            [12, 1, [
                [11, 1, authSessionId]
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            'connectEapAccount', params, self.AAFEC_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_EAP_ENDPOINT, sqrd, self.AAFEC_RES_TYPE)

    def disconnectEapAccount(self, eapType: int = 3):
        params = [
            [12, 1, [
                [8, 1, eapType]
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            'disconnectEapAccount', params, self.AAFEC_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_EAP_ENDPOINT, sqrd, self.AAFEC_RES_TYPE)

    def getHashedPpidForYahoojapan(self):
        params = [
            [12, 1, []]
        ]
        sqrd = self.generateDummyProtocol(
            'getHashedPpidForYahoojapan', params, self.AAFEC_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_EAP_ENDPOINT, sqrd, self.AAFEC_RES_TYPE)

    def openAAFECSession(self, udid: str, deviceModel: str = "Pixel 2"):
        params = [
            [12, 1, [
                [12, 1, [
                    [11, 1, udid],  # len 32
                    [11, 1, deviceModel]
                ]]
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            'openSession', params, self.AAFEC_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_EAP_ENDPOINT, sqrd, self.AAFEC_RES_TYPE)

    def verifyEapLogin(self, authSessionId: str, type: int, accessToken: str):
        """
        - type:
            UNKNOWN(0),
            FACEBOOK(1),
            APPLE(2),
            YAHOOJAPAN(3);
        """
        params = [
            [12, 1, [
                [11, 1, authSessionId],
                [12, 2, [
                    [8, 1, type],
                    [11, 2, accessToken]
                ]]
            ]]
        ]
        sqrd = self.generateDummyProtocol(
            'verifyEapLogin', params, self.AAFEC_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_EAP_ENDPOINT, sqrd, self.AAFEC_RES_TYPE)



# -*- coding: utf-8 -*- 
from .hksc.types import HookTypes
from .hksc.events import HookEvents
from .hksc.utility import HookUtility
from .hksc.database import JsonDatabase
import threading

class HooksTracer(HookTypes, HookUtility):
    
    HooksType = {
        'Operation': 0,
        'Content': 1,
        'Command': 2
    }

    def __init__(self, cl, db=None, prefixes=["!"], accounts=[]):
        self.cl = cl
        self.db = JsonDatabase(cl, db)
        self.prefixes = prefixes
        self.opFuncs = []
        self.contFuncs = []
        self.cmdFuncs = []
        self.beforeFuncs = []
        self.afterFuncs = []
        self.eventFuncs = HookEvents()
        self.accounts = accounts

    def run(self):
        self.eventFuncs.onReady()
        for _cl in self.accounts:
            _td = threading.Thread(target=self.runByClient, args=(_cl, ))
            _td.daemon = True
            _td.start()
        self.runByClient(self.cl)

    def runByClient(self, cl):
        while cl.is_login:
            for op in cl._Poll__fetchOps():
                _td = threading.Thread(target=self.trace, args=(op, self.HooksType['Operation'], cl))
                _td.daemon = True
                _td.start()

    def trace(self, data, type, cl):
        if type == self.HooksType['Operation']:
            _a = self.opFuncs
        elif type == self.HooksType['Content']:
            _a = self.contFuncs
        elif type == self.HooksType['Command']:
            _a = self.cmdFuncs
        else:
            raise Exception(f"unknow type: {type}")
        _b = self.beforeFuncs
        _c = self.afterFuncs
        for _before in _b:
            if _before.type == type:
                if _before(self, data, cl):
                    break
        for _func in _a:
            if _func(self, data, cl):
                return True
        for _after in _c:
            if _after.type == type:
                if _after(self, data, cl):
                    break
        return False


# -*- coding: utf-8 -*-

class SettingsService(object):

    def __init__(self):
        pass

    def getSetting(self):
        params = []
        sqrd = self.generateDummyProtocol('getSetting', params, 4)
        return self.postPackDataAndGetUnpackRespData("/US4", sqrd, 4)

    def contextAgnosticGetSetting(self):
        params = []
        sqrd = self.generateDummyProtocol(
            'contextAgnosticGetSetting', params, 4)
        return self.postPackDataAndGetUnpackRespData("/US4", sqrd, 4)

    def setSetting(self):
        params = []
        sqrd = self.generateDummyProtocol('setSetting', params, 4)
        return self.postPackDataAndGetUnpackRespData("/US4", sqrd, 4)

    def setSettingWithScope(self):
        params = []
        sqrd = self.generateDummyProtocol('setSettingWithScope', params, 4)
        return self.postPackDataAndGetUnpackRespData("/US4", sqrd, 4)

    def resetSetting(self):
        params = []
        sqrd = self.generateDummyProtocol('resetSetting', params, 4)
        return self.postPackDataAndGetUnpackRespData("/US4", sqrd, 4)

    def resetSettingWithScope(self):
        params = []
        sqrd = self.generateDummyProtocol('resetSettingWithScope', params, 4)
        return self.postPackDataAndGetUnpackRespData("/US4", sqrd, 4)

    def searchSettings(self):
        params = []
        sqrd = self.generateDummyProtocol('searchSettings', params, 4)
        return self.postPackDataAndGetUnpackRespData("/US4", sqrd, 4)

    def contextAgnosticSearchSettings(self):
        params = []
        sqrd = self.generateDummyProtocol(
            'contextAgnosticSearchSettings', params, 4)
        return self.postPackDataAndGetUnpackRespData("/US4", sqrd, 4)

    def bulkGetSetting(self, settingItems: list = ["sticker.preview"]):
        settings = []
        for i in settingItems:
            settings.append([
                [11, 1, i]
            ])
        params = [
            [12, 2, [
                [14, 1, [12, settings]]
            ]]
        ]
        sqrd = self.generateDummyProtocol('bulkGetSetting', params, 4)
        return self.postPackDataAndGetUnpackRespData("/US4", sqrd, 4)

    def bulkSetSetting(self, settingItems: list = ["sticker.preview"], TypedValueItems: list = []):
        """
        - TypedValue:
            ("booleanValue", 2, 1);
            ("i64Value", 10, 2);
            ("stringValue", 11, 3);
            ("stringListValue", 15, 4);
            ("i64ListValue", 15, 5);
            ("rawJsonStringValue", 11, 6);
            ("i8Value", 3, 7);
            ("i16Value", 6, 8);
            ("i32Value", 8, 9);
            ("doubleValue", 4, 10);
            ("i8ListValue", 15, 11);
            ("i16ListValue", 15, 12);
            ("i32ListValue", 15, 13);

        example:
            self.bulkSetSetting(
                ['hometab.service.pinned'], 
                [
                    [15, 13, [8, [1419, 1072, 10]]] 
                ]
            )

            what is [15, 13, [8, [1419, 1072, 10]]]?
            that is "i32ListValue", so it using 15 and 13
            then u can see "i32List", so values is a list (type: 15)
            so the data should be [15, 13, [8, values]]
        """
        settings = []
        c = 0
        for i in settingItems:
            settings.append([
                [11, 1, i],
                [12, 2, [TypedValueItems[c]]],
                [10, 3, 0],  # clientTimestampMillis
                # [8, 4, 0],  # ns
                # [11, 5, 0],  # transactionId
            ])
            c += 1
        params = [
            [12, 2, [
                [14, 1, [12, settings]]
            ]]
        ]
        sqrd = self.generateDummyProtocol('bulkGetSetting', params, 4)
        return self.postPackDataAndGetUnpackRespData("/US4", sqrd, 4)



# -*- coding: utf-8 -*-
import time
import json
import requests


class AuthService(object):

    def __init__(self):
        pass

    def openAuthSession(self):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('openAuthSession') + [0, 0, 0, 0]
        sqrd += [12, 0, 2]
        sqrd += [0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT, sqrd)

    def getAuthRSAKey(self, authSessionId: str):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('getAuthRSAKey') + [0, 0, 0, 0]
        sqrd += [11, 0, 2] + self.getStringBytes(authSessionId)
        sqrd += [8, 0, 3] + self.getIntBytes(1)  # identityProvider
        sqrd += [0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT, sqrd)

    def setIdentifier(self, authSessionId: str, cipherKeyId: str, cipherText: str):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('setIdentifier') + [0, 0, 0, 0]
        sqrd += [11, 0, 2] + self.getStringBytes(authSessionId)
        sqrd += [12, 0, 3]
        sqrd += [8, 0, 2] + self.getIntBytes(1)  # identityProvider
        # cipherKeyId, eg.10031
        sqrd += [11, 0, 3] + self.getStringBytes(cipherKeyId)
        # cipherText, rsa enc maybe
        sqrd += [11, 0, 4] + self.getStringBytes(cipherText)
        sqrd += [0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT, sqrd)

    def updateIdentifier(self, authSessionId: str, cipherKeyId: str, cipherText: str):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('updateIdentifier') + [0, 0, 0, 0]
        sqrd += [11, 0, 2] + self.getStringBytes(authSessionId)
        sqrd += [12, 0, 3]
        sqrd += [8, 0, 2] + self.getIntBytes(1)  # identityProvider
        # cipherKeyId, eg.10031
        sqrd += [11, 0, 3] + self.getStringBytes(cipherKeyId)
        # cipherText, rsa enc maybe
        sqrd += [11, 0, 4] + self.getStringBytes(cipherText)
        sqrd += [0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT, sqrd)

    def resendIdentifierConfirmation(self, authSessionId: str):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('resendIdentifierConfirmation') + [0, 0, 0, 0]
        sqrd += [11, 0, 2] + self.getStringBytes(authSessionId)
        sqrd += [12, 0, 3]
        sqrd += [0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT, sqrd)

    def confirmIdentifier(self, authSessionId: str, verificationCode: str):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('confirmIdentifier') + [0, 0, 0, 0]
        sqrd += [11, 0, 2] + self.getStringBytes(authSessionId)
        sqrd += [12, 0, 3]
        sqrd += [12, 0, 5]
        # sqrd += [2, 0, 2] + self.getIntBytes(1) #forceRegistration : 1?
        # verificationCode
        sqrd += [11, 0, 3] + self.getStringBytes(verificationCode)
        sqrd += [0, 0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT, sqrd)

    def removeIdentifier(self, authSessionId: str, cipherKeyId: str, cipherText: str):
        sqrd = [128, 1, 0, 1] + \
            self.getStringBytes('removeIdentifier') + [0, 0, 0, 0]
        sqrd += [11, 0, 2] + self.getStringBytes(authSessionId)
        sqrd += [12, 0, 3]
        sqrd += [8, 0, 2] + self.getIntBytes(1)  # identityProvider
        sqrd += [11, 0, 3] + self.getStringBytes(cipherKeyId)  # cipherKeyId
        sqrd += [11, 0, 4] + self.getStringBytes(cipherText)  # cipherText
        sqrd += [0, 0]
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT, sqrd)

    def getClovaAppToken(self, authSessionId, cipherText, metaData={}):
        params = [
            [11, 2, authSessionId],
            [12, 3, [
                [8, 1, 2],
                [13, 2, [11, 11, metaData]],
                [11, 3, cipherText]
            ]]
        ]
        sqrd = self.generateDummyProtocol('getClovaAppToken', params, 4)
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT_V4, sqrd, 4)

    def loginFromClova(self, authSessionId, cipherText, metaData={}):
        params = [
            [11, 2, authSessionId],
            [12, 3, [
                [8, 1, 2],
                [13, 2, [11, 11, metaData]],
                [11, 3, cipherText]
            ]]
        ]
        sqrd = self.generateDummyProtocol('loginFromClova', params, 4)
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT_V4, sqrd, 4)

    def validateClovaRequest(self, authSessionId, cipherText, metaData={}):
        params = [
            [11, 2, authSessionId],
            [12, 3, [
                [8, 1, 2],
                [13, 2, [11, 11, metaData]],
                [11, 3, cipherText]
            ]]
        ]
        sqrd = self.generateDummyProtocol('validateClovaRequest', params, 4)
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT_V4, sqrd, 4)

    def setClovaCredential(self, authSessionId, cipherText, metaData={}):
        params = [
            [11, 2, authSessionId],
            [12, 3, [
                [8, 1, 2],  # authLoginVersion
                [13, 2, [11, 11, metaData]],
                [11, 3, cipherText]
            ]]
        ]
        sqrd = self.generateDummyProtocol('setClovaCredential', params, 4)
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT_V4, sqrd, 4)

    def validateClovaAppToken(self, authSessionId, cipherText, metaData={}):
        params = [
            [11, 2, authSessionId],
            [12, 3, [
                [8, 1, 2],  # authLoginVersion
                [13, 2, [11, 11, metaData]],
                [11, 3, cipherText]
            ]]
        ]
        sqrd = self.generateDummyProtocol('validateClovaAppToken', params, 4)
        return self.postPackDataAndGetUnpackRespData(self.LINE_AUTH_ENDPOINT_V4, sqrd, 4)

    def verifyQrcodeWithE2EE(self, verifier, keyId, keyData, createdTime, encryptedKeyChain, hashKeyChain, pinCode=""):
        params = [
            [11, 2, verifier],
            [11, 3, pinCode],
            [8, 4, 95],  # errorCode
            [12, 5, [
                [8, 1, 1],  # version
                [8, 2, keyId],
                [11, 4, keyData],
                [10, 5, createdTime]
            ]],
            [11, 6, encryptedKeyChain],
            [11, 7, hashKeyChain]
        ]
        sqrd = self.generateDummyProtocol('verifyQrcodeWithE2EE', params, 4)
        return self.postPackDataAndGetUnpackRespData("/S4", sqrd, 4)

    def confirmE2EELogin(self, verifier, deviceSecret):
        params = [
            [11, 1, verifier],
            [11, 2, deviceSecret],
        ]
        sqrd = self.generateDummyProtocol('confirmE2EELogin', params, 3)
        return self.postPackDataAndGetUnpackRespData("/api/v3p/rs", sqrd, 3)

    def issueV3TokenForPrimary(self, udid: str, systemDisplayName: str, modelName: str):
        params = [
            [12, 1, [
                [11, 1, udid],
                [11, 2, systemDisplayName],
                [11, 3, modelName]
            ]]
        ]
        sqrd = self.generateDummyProtocol('issueV3TokenForPrimary', params, 3)
        return self.postPackDataAndGetUnpackRespData("/api/v3p/rs", sqrd, 3)

    def logoutZ(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("logoutZ is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "logoutZ", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)

    def issueTokenForAccountMigration(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("issueTokenForAccountMigration is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "issueTokenForAccountMigration", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)

    def updatePassword(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("updatePassword is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "updatePassword", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)

    def respondE2EELoginRequest(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("respondE2EELoginRequest is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "respondE2EELoginRequest", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)

    def logoutV2(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("logoutV2 is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "logoutV2", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)

    def establishE2EESession(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("establishE2EESession is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "establishE2EESession", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)

    def releaseLockScreen(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("releaseLockScreen is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "releaseLockScreen", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)

    def normalizePhoneNumber(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("normalizePhoneNumber is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "normalizePhoneNumber", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)

    def exchangeKey(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("exchangeKey is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "exchangeKey", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)

    def setIdentifierAndPassword(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("setIdentifierAndPassword is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "setIdentifierAndPassword", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)

    def issueTokenForAccountMigrationSettings(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception(
            "issueTokenForAccountMigrationSettings is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "issueTokenForAccountMigrationSettings", params, AuthService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(AuthService_API_PATH, sqrd, AuthService_RES_TYPE)



# -*- coding: utf-8 -*-
"""
Author: YinMo
Version: 2.1.0
Description: support pm and group chat!
"""
import hashlib
import json
import os
import base64
import axolotl_curve25519 as Curve25519
from cryptography.hazmat.primitives.ciphers.aead import AESGCM
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad


class E2EE():

    def getE2EELocalPublicKey(self, mid, keyId):
        fd = '.e2eePublicKeys'
        fn = f"key_id_{keyId}.json"
        key = self.getCacheData(fd, fn, False)
        if key is None:
            toType = self.getToType(mid)
            if toType == 0:
                receiver_key_data = self.negotiateE2EEPublicKey(mid)
                if receiver_key_data[3] == -1:
                    raise Exception(f'Not support E2EE on {mid}')
                receiverKeyId = receiver_key_data[2][2]
                receiverKeyData = receiver_key_data[2][4]
                if receiverKeyId == keyId:
                    key = base64.b64encode(receiverKeyData)
                    self.saveCacheData(fd, fn, key.decode(), False)
                else:
                    raise Exception(f'E2EE key id-{keyId} not found on {mid}')
            else:
                fd = '.e2eeGroupKeys'
                fn = f"{mid}.json"
                key = self.getCacheData(fd, fn, False)
                if keyId is not None and key is not None:
                    keyData = json.loads(key)
                    if keyId != keyData['keyId']:
                        self.log(f'keyId mismatch: {mid}')
                        key = None
                if key is None:
                    E2EEGroupSharedKey = self.getLastE2EEGroupSharedKey(
                        2, mid)
                    groupKeyId = E2EEGroupSharedKey[2]
                    creator = E2EEGroupSharedKey[3]
                    creatorKeyId = E2EEGroupSharedKey[4]
                    receiver = E2EEGroupSharedKey[5]
                    receiverKeyId = E2EEGroupSharedKey[6]
                    encryptedSharedKey = E2EEGroupSharedKey[7]
                    selfKey = base64.b64decode(
                        self.getE2EESelfKeyDataByKeyId(receiverKeyId)['privKey'])
                    creatorKey = self.getE2EELocalPublicKey(
                        creator, creatorKeyId)
                    aesKey = self.generateSharedSecret(selfKey, creatorKey)
                    aes_key = self.getSHA256Sum(aesKey, b'Key')
                    aes_iv = self._xor(self.getSHA256Sum(aesKey, b'IV'))
                    aes = AES.new(aes_key, AES.MODE_CBC, aes_iv)
                    decrypted = unpad(aes.decrypt(encryptedSharedKey), 16)
                    self.log(
                        f'[getE2EELocalPublicKey] decrypted: {decrypted}', True)
                    data = {
                        'privKey': base64.b64encode(decrypted).decode(),
                        'keyId': groupKeyId
                    }
                    key = json.dumps(data)
                    self.saveCacheData(fd, fn, key, False)
                return json.loads(key)
        return base64.b64decode(key)

    def generateSharedSecret(self, private_key, public_key):
        return Curve25519.calculateAgreement(bytes(private_key), bytes(public_key))

    def _xor(self, buf):
        buf_length = int(len(buf) / 2)
        buf2 = bytearray(buf_length)
        for i in range(buf_length):
            buf2[i] = buf[i] ^ buf[buf_length + i]
        return bytes(buf2)

    def getSHA256Sum(self, *args):
        instance = hashlib.sha256()
        for arg in args:
            if isinstance(arg, str):
                arg = arg.encode()
            instance.update(arg)
        return instance.digest()

    def _encryptAESECB(self, aes_key, plain_data):
        aes = AES.new(aes_key, AES.MODE_ECB)
        return aes.encrypt(plain_data)

    def decryptKeyChain(self, publicKey, privateKey, encryptedKeyChain):
        shared_secret = self.generateSharedSecret(privateKey, publicKey)
        aes_key = self.getSHA256Sum(shared_secret, 'Key')
        aes_iv = self._xor(self.getSHA256Sum(shared_secret, 'IV'))
        aes = AES.new(aes_key, AES.MODE_CBC, aes_iv)
        keychain_data = aes.decrypt(encryptedKeyChain)
        key = keychain_data.hex()
        key = bin2bytes(key)
        tc = self.TCompactProtocol(passProtocol=True)
        tc.data = key
        key = tc.x(False)[1]
        public_key = bytes(key[0][4])
        private_key = bytes(key[0][5])
        return [private_key, public_key]

    def encryptDeviceSecret(self, publicKey, privateKey, encryptedKeyChain):
        shared_secret = self.generateSharedSecret(privateKey, publicKey)
        aes_key = self.getSHA256Sum(shared_secret, 'Key')
        encryptedKeyChain = self._xor(self.getSHA256Sum(encryptedKeyChain))
        keychain_data = self._encryptAESECB(aes_key, encryptedKeyChain)
        return keychain_data

    def generateAAD(self, a, b, c, d, e=2, f=0):
        aad = b''
        aad += a.encode()
        aad += b.encode()
        aad += bytes(self.getIntBytes(c))
        aad += bytes(self.getIntBytes(d))
        aad += bytes(self.getIntBytes(e))  # e2ee version
        aad += bytes(self.getIntBytes(f))  # content type
        return aad

    def encryptE2EEMessage(self, to, text, specVersion=2, isCompact=False):
        _from = self.mid
        selfKeyData = self.getE2EESelfKeyData(_from)
        if len(to) == 0 or self.getToType(to) not in [0, 1, 2]:
            raise Exception('Invalid mid')
        if selfKeyData is None:
            raise Exception(
                'E2EE Key has not been saved, try register or use SQR Login')
        senderKeyId = selfKeyData['keyId']
        if self.getToType(to) == 0:
            private_key = base64.b64decode(selfKeyData['privKey'])
            receiver_key_data = self.negotiateE2EEPublicKey(to)
            if receiver_key_data[3] == -1:
                raise Exception(f'Not support E2EE on {to}')
            receiverKeyId = receiver_key_data[2][2]
            keyData = self.generateSharedSecret(
                bytes(private_key), receiver_key_data[2][4])
            specVersion = receiver_key_data[3]
        else:
            groupK = self.getE2EELocalPublicKey(to, None)
            privK = base64.b64decode(groupK['privKey'])
            pubK = base64.b64decode(selfKeyData['pubKey'])
            receiverKeyId = groupK['keyId']
            keyData = self.generateSharedSecret(privK, pubK)
        chunks = self.encryptE2EETextMessage(
            senderKeyId, receiverKeyId, keyData, specVersion, text, to, _from, isCompact=isCompact)
        return chunks

    def encryptE2EETextMessage(self, senderKeyId, receiverKeyId, keyData, specVersion, text, to, _from, isCompact=False):
        salt = os.urandom(16)
        gcmKey = self.getSHA256Sum(keyData, salt, b'Key')
        aad = self.generateAAD(to, _from, senderKeyId,
                               receiverKeyId, specVersion, 0)
        sign = os.urandom(16)
        data = json.dumps({
            'text': text
        }).encode()
        encData = self.encryptE2EEMessageV2(data, gcmKey, sign, aad)
        bSenderKeyId = bytes(self.getIntBytes(senderKeyId))
        bReceiverKeyId = bytes(self.getIntBytes(receiverKeyId))
        if isCompact:
            compact = self.TCompactProtocol()
            bSenderKeyId = bytes(compact.writeI32(int(senderKeyId)))
            bReceiverKeyId = bytes(compact.writeI32(int(receiverKeyId)))
        self.log(
            f'senderKeyId: {senderKeyId} ({bSenderKeyId})', True)
        self.log(
            f'receiverKeyId: {receiverKeyId} ({bReceiverKeyId})', True)
        return [salt, encData, sign, bSenderKeyId, bReceiverKeyId]

    def encryptE2EEMessageV2(self, data, gcmKey, nonce, aad):
        aesgcm = AESGCM(gcmKey)
        return aesgcm.encrypt(nonce, data, aad)

    def decryptE2EETextMessage(self, messageObj, isSelf=True):
        _from = messageObj[1]
        to = messageObj[2]
        toType = messageObj[3]
        metadata = messageObj[18]
        specVersion = metadata.get('e2eeVersion', '2')
        contentType = metadata.get('contentType', '0')
        chunks = messageObj[20]
        for i in range(len(chunks)):
            if isinstance(chunks[i], str):
                chunks[i] = chunks[i].encode()
        senderKeyId = byte2int(chunks[3])
        receiverKeyId = byte2int(chunks[4])
        self.log(f'senderKeyId: {senderKeyId}', True)
        self.log(f'receiverKeyId: {receiverKeyId}', True)

        selfKey = self.getE2EESelfKeyData(self.mid)
        privK = base64.b64decode(selfKey['privKey'])
        if toType == 0:
            pubK = self.getE2EELocalPublicKey(
                to, receiverKeyId if isSelf else senderKeyId)
        else:
            groupK = self.getE2EELocalPublicKey(to, receiverKeyId)
            privK = base64.b64decode(groupK['privKey'])
            pubK = base64.b64decode(selfKey['pubKey'])
            if _from != self.mid:
                pubK = self.getE2EELocalPublicKey(_from, senderKeyId)

        if specVersion == '2':
            decrypted = self.decryptE2EEMessageV2(
                to, _from, chunks, privK, pubK, specVersion, contentType)
        else:
            decrypted = self.decryptE2EEMessageV1(chunks, privK, pubK)
        return decrypted.get('text', '')

    def decryptE2EEMessageV1(self, chunks, privK, pubK):
        salt = chunks[0]
        message = chunks[1]
        sign = chunks[2]
        aesKey = self.generateSharedSecret(privK, pubK)
        aes_key = self.getSHA256Sum(aesKey, salt, b'Key')
        aes_iv = self._xor(self.getSHA256Sum(aesKey, salt, 'IV'))
        aes = AES.new(aes_key, AES.MODE_CBC, aes_iv)
        decrypted = aes.decrypt(message)
        self.log(f'decrypted: {decrypted}', True)
        decrypted = unpad(decrypted, 16)
        return json.loads(decrypted)

    def decryptE2EEMessageV2(self, to, _from, chunks, privK, pubK, specVersion=2, contentType=0):
        salt = chunks[0]
        message = chunks[1]
        sign = chunks[2]
        senderKeyId = byte2int(chunks[3])
        receiverKeyId = byte2int(chunks[4])

        aesKey = self.generateSharedSecret(privK, pubK)
        gcmKey = self.getSHA256Sum(aesKey, salt, b'Key')
        aad = self.generateAAD(to, _from, senderKeyId,
                               receiverKeyId, specVersion, contentType)

        aesgcm = AESGCM(gcmKey)
        decrypted = aesgcm.decrypt(sign, message, aad)
        self.log(f'decrypted: {decrypted}', True)
        return json.loads(decrypted)


def byte2int(t):
    e = 0
    i = 0
    s = len(t)
    for i in range(s):
        e = 256 * e + t[i]
    return e


def bin2bytes(k):
    e = []
    for i in range(int(len(k) / 2)):
        _i = int(k[i * 2:i * 2 + 2], 16)
        e.append(_i)
    return bytearray(e)



# -- coding utf-8 --

class SearchService(object):

    def __init__(self):
        pass

    def searchAll(self):
        params = []
        sqrd = self.generateDummyProtocol('searchAll', params, 4)
        return self.postPackDataAndGetUnpackRespData("/search/v3", sqrd, 4)

    def searchCollection(self):
        params = []
        sqrd = self.generateDummyProtocol('searchCollection', params, 4)
        return self.postPackDataAndGetUnpackRespData("/search/v3", sqrd, 4)

    def searchLineat(self):
        params = []
        sqrd = self.generateDummyProtocol('searchLineat', params, 4)
        return self.postPackDataAndGetUnpackRespData("/search/v3", sqrd, 4)

    def searchByPopularCategory(self):
        params = []
        sqrd = self.generateDummyProtocol('searchByPopularCategory', params, 4)
        return self.postPackDataAndGetUnpackRespData("/search/v3", sqrd, 4)

    def searchByCategory(self):
        params = []
        sqrd = self.generateDummyProtocol('searchByCategory', params, 4)
        return self.postPackDataAndGetUnpackRespData("/search/v3", sqrd, 4)

    def getPopularCategory(self):
        params = []
        sqrd = self.generateDummyProtocol('getPopularCategory', params, 4)
        return self.postPackDataAndGetUnpackRespData("/search/v3", sqrd, 4)

    def getNotice(self):
        params = []
        sqrd = self.generateDummyProtocol('getNotice', params, 4)
        return self.postPackDataAndGetUnpackRespData("/search/v3", sqrd, 4)

    def getSearchSection(self):
        params = []
        sqrd = self.generateDummyProtocol('getSearchSection', params, 4)
        return self.postPackDataAndGetUnpackRespData("/search/v3", sqrd, 4)

    def getAutocomplete(self):
        params = []
        sqrd = self.generateDummyProtocol('getAutocomplete', params, 4)
        return self.postPackDataAndGetUnpackRespData("/search/v3", sqrd, 4)

    def searchAll(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("searchAll is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "searchAll", params, SearchService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(SearchService_API_PATH, sqrd, SearchService_RES_TYPE)

    def searchInContext(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("searchInContext is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "searchInContext", params, SearchService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(SearchService_API_PATH, sqrd, SearchService_RES_TYPE)

    def getPopularCategory(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getPopularCategory is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getPopularCategory", params, SearchService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(SearchService_API_PATH, sqrd, SearchService_RES_TYPE)

    def getAutocomplete(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getAutocomplete is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getAutocomplete", params, SearchService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(SearchService_API_PATH, sqrd, SearchService_RES_TYPE)

    def searchByPopularCategory(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("searchByPopularCategory is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "searchByPopularCategory", params, SearchService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(SearchService_API_PATH, sqrd, SearchService_RES_TYPE)

    def searchByCategory(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("searchByCategory is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "searchByCategory", params, SearchService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(SearchService_API_PATH, sqrd, SearchService_RES_TYPE)

    def searchLineat(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("searchLineat is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "searchLineat", params, SearchService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(SearchService_API_PATH, sqrd, SearchService_RES_TYPE)

    def getNotice(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getNotice is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getNotice", params, SearchService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(SearchService_API_PATH, sqrd, SearchService_RES_TYPE)

    def getSearchSection(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("getSearchSection is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "getSearchSection", params, SearchService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(SearchService_API_PATH, sqrd, SearchService_RES_TYPE)

    def searchCollection(self):
        """
        AUTO_GENERATED_CODE! DONT_USE_THIS_FUNC!!
        """
        raise Exception("searchCollection is not implemented")
        params = []
        sqrd = self.generateDummyProtocol(
            "searchCollection", params, SearchService_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(SearchService_API_PATH, sqrd, SearchService_RES_TYPE)



# -*- coding: utf-8 -*-

class ChatAppService(object):
    CAPP_REQ_TYPE = 4
    CAPP_RES_TYPE = 4
    
    def __init__(self):
        pass

    def getChatapp(self, chatappId: str, language: str = 'zh_TW'):
        params = [
            [12, 1, [
                [11, 1, chatappId],
                [11, 2, language]
            ]]
        ]
        sqrd = self.generateDummyProtocol('getChatapp', params, self.CAPP_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_CHAT_APP_ENDPOINT, sqrd, self.CAPP_RES_TYPE)

    def getMyChatapps(self, language: str = 'zh_TW', continuationToken: str = None):
        params = [
            [12, 1, [
                [11, 1, language],
                [11, 2, None]
            ]]
        ]
        sqrd = self.generateDummyProtocol('getMyChatapps', params, self.CAPP_REQ_TYPE)
        return self.postPackDataAndGetUnpackRespData(self.LINE_CHAT_APP_ENDPOINT, sqrd, self.CAPP_RES_TYPE)